#!/usr/bin/env python
"""
Add or update the license information in the header of source files.

The script reads in the .gitattributes file, located in the project root
directory, to figure out which files need to be inspected and which license
header they need to have.
For example:

src/pedra/pedra_dlapack.F90 !licensefile
src/solver/*.hpp licensefile=.githooks/LICENSE-C++

The first line specifies that the file in src/pedra/pedra_dlapack.F90 should
not be touched, while the second line states that all .hpp files in src/solver
should get an header from the template in .githooks/LICENSE-C++
Location of files in .gitattributes are always specified with respect
to the project root directory.

The script reads in the appropriate license header template and prepares an
header with the correct year and authors information. These are encoded in the
variables YEAR and AUTHORS. The latter has to be modified by hand.
"""

import glob
import os
import re
import shutil
import sys
import tempfile
from datetime import date


def add_header(filepath, header, TEXT, YEAR, AUTHORS):
    """
    Add or update header in source file
    """
    tmpdir = tempfile.gettempdir()
    tmpfil = os.path.join(tmpdir, os.path.basename(filepath) + '.bak')
    shutil.copy2(filepath, tmpfil)
    with open(tmpfil, 'r') as tmp:
        inpt = tmp.readlines()
        output = []

        # Check if header is already present
        present = re.compile(TEXT)
        if list(filter(present.search, inpt)):
            # Check if year and authors in current file are up to date
            toupdate = re.compile(r'{0:s} (?!{1:d} {2:s}).*\n'.format(
                'Copyright \(C\)', YEAR, AUTHORS))
            if list(filter(toupdate.search, inpt)):
                print(('Updating header in {:s}'.format(filepath)))
                # Check to preserve '#!' at the top of the file
                if len(inpt) > 0 and inpt[0].startswith('#!'):
                    output.append('{}\n'.format(inpt[0]))
                    inpt = inpt[1:]
                regex = re.compile(r'Copyright \(C\).*\n')
                repl = r'Copyright (C) {0:d} {1:s}\n'.format(YEAR, AUTHORS)
                output.extend([re.sub(regex, repl, x) for x in inpt])
        else:
            print(('Adding header in {:s}'.format(filepath)))
            # Check to preserve '#!' at the top of the file
            if len(inpt) > 0 and inpt[0].startswith('#!'):
                output.append('{:s}\n'.format(inpt[0]))
                inpt = inpt[1:]
            output.append('{:s}\n\n'.format(header))
            for line in inpt:
                output.append(line)

        if output:
            try:
                f = open(filepath, 'w')
                f.writelines(output)
            except IOError as err:
                print(('Something went wrong trying to add header to {:s}: {:s}'.
                       format(filepath, err)))
            finally:
                f.close()
        os.remove(tmpfil)


def prepare_header(stub, YEAR, AUTHORS):
    """
    Update year and author information in license header template
    """
    with open(stub, 'r') as l:
        header = l.read().rstrip()
        # Insert correct YEAR and AUTHORS in stub
        rep = {'YEAR': str(YEAR), 'AUTHORS': AUTHORS}
        rep = dict((re.escape(k), v) for k, v in rep.items())
        pattern = re.compile("|".join(list(rep.keys())))
        header = pattern.sub(lambda m: rep[re.escape(m.group(0))], header)
    return header


def file_license(attributes):
    """
    Obtain dictionary { file : license } from .gitattributes
    """
    file_license = {}
    with open(attributes, 'r') as f:
        # Read in .gitattributes
        tmp = f.read()
        # Removing all comment lines and other attributes
        pattern = re.compile(r'(?m)^\#.*\n?|^((?!licensefile).)*$')
        gitattributes = re.sub(pattern, '', tmp).split()
        # Obtain list of files
        fil = [x for x in gitattributes if not 'licensefile' in x]
        # Remove licensefile= from strings
        lic = [
            re.sub(r'licensefile\=', '', x) for x in gitattributes
            if 'licensefile' in x
        ]
        # Create list of blacklisted files
        blacklist = [
            fname for key, value in list(dict(list(zip(fil, lic))).items())
            if value == '!licensefile' for fname in glob.glob(key)
        ]
        # Now create a dictionary with the files to be considered for
        # license header manipulation
        file_license = {
            key: value
            for k, value in list(dict(list(zip(fil, lic))).items())
            for key in glob.glob(k) if key not in blacklist
        }
    return file_license


def license_maintainer(TEXT, AUTHORS):
    """
    Maintain license header in source files
    """
    YEAR = date.today().year

    script_dir = os.path.dirname(os.path.abspath(__file__))
    project_root_dir = os.path.abspath(os.path.join(script_dir, os.pardir))

    headerize = file_license(os.path.join(project_root_dir, '.gitattributes'))

    for fname, license in list(headerize.items()):
        # Prepare header
        header = prepare_header(
            os.path.join(project_root_dir, license), YEAR, AUTHORS)
        add_header(fname, header, TEXT, YEAR, AUTHORS)


if __name__ == '__main__':
    TEXT = sys.argv[1]
    AUTHORS = sys.argv[2]
    license_maintainer(TEXT, AUTHORS)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""The setup script."""

from setuptools import find_packages, setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'Click>=6.0',
    'pyparsing>=2.2',
]

setup_requirements = [
    'pytest-runner',
]

test_requirements = [
    'pytest',
]

setup(
    author="Roberto Di Remigio",
    author_email='roberto.diremigio@gmail.com',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    description="Generic input parsing library, speaking in tongues",
    entry_points={
        'console_scripts': [
            'parselglossy=parselglossy.cli:main',
        ],
    },
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='parselglossy',
    name='parselglossy',
    packages=find_packages(include=['parselglossy']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/dev-cafe/parselglossy',
    version='0.1.0',
    zip_safe=False,
)



#
# parselglossy -- Generic input parsing library, speaking in tongues
# Copyright (C) 2018 Roberto Di Remigio, and contributors.
#
# This file is part of parselglossy.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# Roberto Di Remigio, and contributors. OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# For information on the complete list of contributors to the
# parselglossy library, see: <http://parselglossy.readthedocs.io/>
#

# -*- coding: utf-8 -*-
"""Top-level package for parselglossy."""

__author__ = """Roberto Di Remigio"""
__email__ = 'roberto.diremigio@gmail.com'
__version__ = '0.1.0'
__copyright__ = "Copyright 2018, dev-cafe"
__credits__ = [
    "Jonas Juselius", "Roberto Di Remigio", "Radovan Bast", "Luca Frediani",
    "Stig Rune Jensen", "Ville Weijo"
]
__license__ = "MIT"
__maintainer__ = "Roberto Di Remigio, Radovan Bast, Stig Rune Jensen"
__status__ = "Production"



#
# parselglossy -- Generic input parsing library, speaking in tongues
# Copyright (C) 2018 Roberto Di Remigio, and contributors.
#
# This file is part of parselglossy.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# Roberto Di Remigio, and contributors. OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# For information on the complete list of contributors to the
# parselglossy library, see: <http://parselglossy.readthedocs.io/>
#

# -*- coding: utf-8 -*-
"""Console script for parselglossy."""
import sys
import click


@click.command()
def main(args=None):
    """Console script for parselglossy."""
    click.echo("Replace this message by putting your code into "
               "parselglossy.cli.main")
    click.echo("See click documentation at http://click.pocoo.org/")
    return 0


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover
